#include "avl.h"
#include <stdio.h>



// cria uma rvore vazia
Tree CreateTree(void)
{
  return NULL;
}

// testa se a rvore  vazia 
int TreeEmpty(Tree root)
{
  return (root==NULL);
}

// conta o nmero de nodos da rvore
int TreeSize(Tree root)
{
  if (!root)
    return 0;
  else
    return (1+TreeSize(root->left)+TreeSize(root->right));
}

// calcula a altura da rvore
int TreeHeight(Tree root)
{
  int a,b;

  if (root){
    a=TreeHeight(root->left);
    b=TreeHeight(root->right);
    return (a > b ? a+1 : b+1);
  }
  else return 0;
}


// apaga toda rvore
void ClearTree(Tree *root)
{
  Tree l,r;
  if (*root){
    l=(*root)->left;
    r=(*root)->right;
    free(*root);
    ClearTree(&l);
    ClearTree(&r);
    *root=NULL;
  }
}


// travessia preorder
void Preorder(Tree root, Fun f)
{
  if (root){
    f(root);
    Preorder(root->left, f);
    Preorder(root->right, f);
  }
}


// travessia inorder
void Inorder(Tree root, Fun f)
{
  if (root){
    Inorder(root->left, f);
    f(root);
    Inorder(root->right, f);
  }
}


// travessia postorder
void Postorder(Tree root, Fun f)
{
  if (root) {
    Postorder(root->left, f);
    Postorder(root->right, f);
    f(root);
  }
}

// ClearTree implementada como postorder 

void FreeNode(TreeNode *p) 
{
  if (p) free(p);
}


void ClearTreePost(Tree *root) 
{
  Postorder(*root,FreeNode);
  *root=NULL;
}


//==============================================
// RVORES BINRIAS DE PROCURA Balanceadas (AVL)


// procura um elemento numa rvore binria de procura (RECURSIVA)
TreeNode *TreeSearch(Tree root, TreeEntry k)
{
  if (root){
    if ( k < root->entry)
      return TreeSearch(root->left, k);
    else if (k > root->entry)
      return TreeSearch(root->right, k);
  }
  return root;
}



// insere um novo elemento numa rvore binria de procura AVL.
Tree InsertTree(Tree root, TreeEntry k)
{
  int x;
  return InsertAVL(root, k, &x);
}


// insere um novo elemento numa rvore  AVL 
// (o terceiro parametro (Bool*) indica se a a altura da rvore aumentou ou no)
Tree InsertAVL(Tree root, TreeEntry k, Bool *taller)
{
  if (!root) {   // vazia
    root = (Tree)malloc(sizeof(TreeNode));
    root->entry = k;
    root->right = root->left = NULL;
    root->bf = EH;
    *taller = TRUE;
  }
  else if (k > root->entry) {
    root->right = InsertAVL(root->right, k, taller);
    if (*taller)
      switch (root->bf) {
      case LH:                      // caso 2
	root->bf = EH;
	*taller = FALSE;
	break;
      case EH:                      // caso 2
	root->bf = RH;
	//	*taller = TRUE;
	break;
      case RH:                      // caso 3
	root = RightBalance(root);
	*taller = FALSE;
	break;
      }
  }
  else if (k < root->entry) {
    root->left = InsertAVL(root->left, k, taller);
    if (*taller)
      switch (root->bf) {
      case LH:                      // caso 3
	root = LeftBalance(root);
	*taller = FALSE;
	break;
      case EH:                      // caso 2
	root->bf = LH;
	//	*taller = TRUE;
	break;
      case RH:                      // caso 2
	root->bf = EH;
	*taller = FALSE;
	break;
      }
  }
  else  printf("\n Chave %d repetida ... \n", k);

  return root;
}

// balanceamneto da rvore na situao RIGHT-RIGHT-TALLER
Tree RightBalance(Tree root)
{
  Tree rl, r=root->right;

  switch (r->bf) {
  case RH:                      // caso 1   rotao  esquerda

    ...................

    break;
  case LH:                      // caso 2   dupla rotao

    ...................

    break;
  case EH:
    printf("\n Erro: arvore balanceada. \n");
    break;
  }
  return root;
}




// rotao para a direita
Tree RotateRight(Tree root)
{
  Tree l=root;

  if (root && root->left) {

    ...................

  }
  else printf("\n Erro ... \n");

  return l;
}

// rotao para a esquerda
Tree RotateLeft(Tree root)
{
  Tree r=root;

  if (root && root->right) {

    ...................

  }
  else printf("\n Erro ... \n");

  return r;
}







// balanceamneto da rvore na situao LEFT-LEFT-TALLER
Tree LeftBalance(Tree root)
{
  Tree l=root->left, lr;

  switch (l->bf) {
  case LH:                      // caso 1   rotao  esquerda
    root->bf = l->bf = EH;
    root = RotateRight(root);
    break;
  case RH:                      // caso 2   dupla rotao
    lr = l->left;
    switch (lr->bf) {
    case RH:
      root->bf = EH;
      l->bf = LH;
      break;
    case LH:
      root->bf = RH;
      l->bf = EH;
      break;
    case EH:                    // s acontece quando Z acaba de ser criado
      root->bf = l->bf = EH;    
      break;
    }                    
    lr->bf = EH;
    root->right = RotateLeft(l);
    root = RotateRight(root);
    break;
  case EH:
    printf("\n Erro: arvore balanceada. \n");
    break;
  }
  return root;
}





